# standard imports
import sys
import logging

# external imports
from cic_eth_registry import CICRegistry
from cic_eth_registry.lookup.tokenindex import TokenIndexLookup
from cic_types.ext.metadata import MetadataRequestsHandler

# local imports
from clicada.tx import TxGetter
from clicada.user import FileUserStore
from clicada.token import FileTokenStore
from clicada.cli import CmdCtrl
from clicada.tx import ResolvedTokenTx

logging.basicConfig(level=logging.WARNING)
logg = logging.getLogger()

ctrl = CmdCtrl(argv=sys.argv[1:], logger=logg)

tx_getter = TxGetter(ctrl.get('TX_CACHE_URL'))

store_path = '.clicada'
user_phone_file_label = 'phone'
user_phone_store = FileUserStore(ctrl.chain(), user_phone_file_label, store_path)
try:
    user_phone_store.put('25413241324', '5c4f9EeE1a6375d30f50ab547cD4EE21B37ac8Ab')
except FileExistsError:
    logg.debug('already have record')

k = '25413241324'
user_address = user_phone_store.get(k)
logg.debug('loaded user address {} for {}'.format(user_address, k))

txs = tx_getter.get(user_address)

token_store = FileTokenStore(ctrl.chain(), ctrl.conn(), 'token', store_path)

MetadataRequestsHandler.base_url = ctrl.get('META_URL')

user_address_file_label = 'address'
user_address_store = FileUserStore(ctrl.chain(), user_address_file_label, store_path)

for tx_src in txs['data']:
    tx = ResolvedTokenTx.from_dict(tx_src)
    tx.resolve(token_store, user_address_store)
    print(tx)
